﻿namespace TabsStudioMvcGroup
{
    public class TabInfo
    {
        public TabInfo(EnvDTE.Window w)
        {
            type = TabType.Unknown;

            if (w.Type != EnvDTE.vsWindowType.vsWindowTypeDocument)
                return;

            string fileName = null, directoryName = null, upperDirectory = null;
            try
            {
                EnvDTE.Document document = w.Document; // can throw
                if (document != null)
                {
                    fileName = System.IO.Path.GetFileName(document.FullName);
                    upperDirectory = System.IO.Path.GetDirectoryName(document.FullName);
                    directoryName = System.IO.Path.GetFileName(upperDirectory);
                    upperDirectory = System.IO.Path.GetDirectoryName(upperDirectory);
                }
            }
            catch (System.Exception)
            {
                return;
            }
            if (fileName == null || directoryName == null || upperDirectory == null)
                return;

            if (string.Compare(directoryName, "Controllers", true) == 0 &&
                fileName.EndsWith(controllerTail, System.StringComparison.CurrentCultureIgnoreCase))
            {
                type = TabType.Controller;
                groupName = fileName.Substring(0, fileName.Length - controllerTail.Length);
                rootDirectory = upperDirectory;
                return;
            }

            if (string.Compare(System.IO.Path.GetFileName(upperDirectory), "Views", true) == 0)
            {
                type = TabType.View;
                groupName = directoryName;
                rootDirectory = System.IO.Path.GetDirectoryName(upperDirectory);
                return;
            }
        }

        public TabInfo()
        {
            type = TabType.Unknown;
        }

        public static bool SameGroup(TabInfo lhs, TabInfo rhs)
        {
            return System.String.Compare(lhs.groupName, rhs.groupName, true) == 0 &&
                   System.String.Compare(lhs.rootDirectory, rhs.rootDirectory, true) == 0;
        }

        public TabType type;
        public string groupName;

        private string rootDirectory;
        private readonly string controllerTail = "Controller.cs";
    }
}
